

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JToggleButton;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Erilaisia nappuloita ja stimi (spinner, slider) esittelev vlilehti.
 * Huomaa, ett vain osaan komponenteista on sidottu ns. erikoistettu kuuntelija.
 * Siis tarkoittaen sit, ett ko kuuntelija kuuntelee vain tietyn tai tiettyjen
 * komponenttien tapahtumia. Muut komponentit reagoivat kyll tapahtumiin, mutta
 * ne liittvt itseens yliluokan tapahtumakuuntelijan. Tm on tarkoituksellinen
 * valinta koska en viitsinyt luoda jokaiselle komponentille omaa kuuntelijaa.
 * Tehtvn keskeisen tarkoituksena kun oli kuitenkin esitell erilaisia Swing-
 * komponentteja sek niiden tapahtumakuuntelua. Ymmrtkseni toteutus kuitenkin
 * vastaa tysin tehtvnannon asettamia vaatimuksia.
 * 
 *
 */
public class ButtonsTab extends ComponentTab {
	
	private JPanel radios;
	private JPanel checkboxes;
	private JPanel toggles;	
	private JPanel buttons;
	private JPanel spinner;
	private JPanel slider;
	
	private JToggleButton tb1; 
	private JToggleButton tb2;
	private JToggleButton tb3; 

	
	public ButtonsTab(SwingComponents parent) {
		super(parent);
	}

	protected void initialize() {
		this.add(getRadioPanel());
		this.add(getCheckboxPanel());
		this.add(getTogglePanel());
		this.add(getButtonPanel());
		this.add(getSpinnerPanel());
		this.add(getSliderPanel());
	}
	
	private JPanel getRadioPanel() {
		if (radios == null) {
			radios = new JPanel();
			radios.setBorder(new TitledBorder("Radio buttons"));
			JRadioButton rb1 = new JRadioButton("YLEX");
			JRadioButton rb2 = new JRadioButton("Radio Helsinki");
			JRadioButton rb3 = new JRadioButton("DI.fm", true);
		
			ButtonGroup group = new ButtonGroup();
			group.add(rb1);
			group.add(rb2);
			group.add(rb3);
			radios.add(rb1);
			radios.add(rb2);
			radios.add(rb3);
			rb1.addActionListener(this);
			rb2.addActionListener(this);
			rb3.addActionListener(this);
			
		}
	
		return radios;
	}
	
	private JPanel getCheckboxPanel() {
		if (checkboxes == null) {
			checkboxes = new JPanel();
			checkboxes.setBorder(new TitledBorder("Checkboxes"));
			JCheckBox cb1 = new JCheckBox("Cat");
			JCheckBox cb2 = new JCheckBox("Dog");
			JCheckBox cb3 = new JCheckBox("Bird");
			cb1.addActionListener(this);
			cb2.addActionListener(this);
			cb3.addActionListener(this);
			
			checkboxes.add(cb1);
			checkboxes.add(cb2);
			checkboxes.add(cb3);
			
		}
		
		return checkboxes;
	}

	private JPanel getTogglePanel() {
		if (toggles == null) {
			toggles = new JPanel();
			toggles.setBorder(new TitledBorder("Toggle buttons"));
			ToggleButtonListener tbl = new ToggleButtonListener();
			
			tb1 = new JToggleButton("Turbo boost");
			tb2 = new JToggleButton("Mega power");
			tb3 = new JToggleButton("God mode");
			
			tb1.addActionListener(tbl);
			tb2.addActionListener(tbl);
			tb3.addActionListener(tbl);
			
			toggles.add(tb1);
			toggles.add(tb2);
			toggles.add(tb3);

		}
	
		return toggles;
	}

	private JPanel getButtonPanel() {
		if (buttons == null) {
			buttons = new JPanel();
			buttons.setBorder(new TitledBorder("Buttons"));
			JButton b1 = new JButton("Detonate");
			JButton b2 = new JButton("Disintegrate");
			JButton b3 = new JButton("Exterminate");
			b1.addActionListener(this);
			b2.addActionListener(this);
			b3.addActionListener(this);
			buttons.add(b1);
			buttons.add(b2);
			buttons.add(b3);
			
			
		}
	
		return buttons;
	}
	
	private JPanel getSpinnerPanel() {
		if (spinner == null) {
			spinner = new JPanel();
			spinner.setBorder(new TitledBorder("Spinner"));
			final JSpinner jsp = new JSpinner();
			jsp.setValue(15000);
			jsp.addChangeListener(new ChangeListener() {
				public void stateChanged(javax.swing.event.ChangeEvent e) {
					parent.getJTextArea().append("Spinner value is "+jsp.getValue()+"\n");
				}
			});
			spinner.add(jsp);
			
		}
		return spinner;
	}

	private JPanel getSliderPanel() {
		if (slider == null) {
			slider = new JPanel();
			slider.setBorder(new TitledBorder("Slider"));
			final JSlider slide = new JSlider();
			slide.setValue(1000);
			slide.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					parent.getJTextArea().append("Slider value is "+slide.getValue()+"\n");
				}
			});
			slider.add(slide);
		}
		return slider;
	}
	
	
	/**
	 * Esimerkki kuuntelijasta yksityisen sisluokkana. Tt kuuntelijaa kytetn
	 * ksittelemn toggle-nappien tapahtumia. Muut komponenttien tapahtumat ksitelln
	 * yliluokan toteuttamassa kuuntelijassa. Tm ratkaisu on tehty siksi, ett on jrjetnt
	 * kirjoittaa jokaiselle mahdolliselle komponentille omat kuuntelijansa. Varsinkin kun
	 * tehtvn pasiallisena tarkoituksena oli esitell erilaisia Swingin kyttliittymkonponentteja.
	 * 
	 * 
	 */
	private class ToggleButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			
			// tss siis reagoidaan nappien tapahtumiin
			if (e.getSource() == tb1) {
				parent.getJTextArea().append("Turbo boost was " + (tb1.isSelected() ? "selected" : "deselected")+"\n");
			} else if (e.getSource() == tb2) {
				parent.getJTextArea().append("Mega power was " + (tb2.isSelected() ? "selected" : "deselected")+"\n");
			} else if (e.getSource() == tb3) {
				parent.getJTextArea().append("God mode was " + (tb3.isSelected() ? "selected" : "deselected")+"\n");
			}
		
		}
	}
}
