

import java.awt.Component;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;



public class SwingComponents extends JFrame {

	private JMenuBar jMenuBar;
	private JMenu fileMenu;
	private JMenu lafMenu;	
	private JMenuItem aboutMenuItem;
	private JMenuItem exitMenuItem;
	private JPopupMenu popupMenu;

	private JSplitPane jSplitPane;
	private JTabbedPane jTabbedPane;
	private JTextArea jTextArea;
	
	
	
	public SwingComponents() {
		super();
		initializeGUI();
		
	}
	
	private void initializeGUI() {
	
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension size = new Dimension(800, 600);		
		this.setLocation( (screen.width - size.width) / 2, (screen.height - size.height) / 2);
		this.setSize(size);
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		this.setTitle("Swing Component Demo");
		this.setJMenuBar(getJMenuBar());
		this.setContentPane(getJSplitPane());
		this.setVisible(true);

	}
	
	public Component getComponent() {
		return this;
	}
	
	public JMenuBar getJMenuBar() {
		if (jMenuBar == null) {
			jMenuBar = new JMenuBar();
			jMenuBar.add(getFileMenu());
			jMenuBar.add(getLafMenu());
		}
		
		return jMenuBar;
	}

	
	
	
	public JMenu getFileMenu() {
		if (fileMenu == null) {
			fileMenu = new JMenu("File");			
			fileMenu.setMnemonic(KeyEvent.VK_F);
			fileMenu.add(getAboutMenuItem());
			fileMenu.addSeparator();
			fileMenu.add(getExitMenuItem());
		}
		return fileMenu;
	}

	
	public JMenu getLafMenu() {
		if (lafMenu == null) {
			lafMenu = new JMenu("Look & Feel");
			lafMenu.setMnemonic(KeyEvent.VK_L);
		
			ButtonGroup group = new ButtonGroup();
			LAFActionListener lafListener = new LAFActionListener(getComponent());
			LookAndFeel laf = UIManager.getLookAndFeel();
			
			// listn valikkoon vain asennetut lafit 
			for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
				JRadioButtonMenuItem item = new JRadioButtonMenuItem(info.getName(), laf.getName().equals(info.getName()));								
				item.addActionListener(lafListener);
				item.setActionCommand(info.getClassName());	// vlitetn lafin toteuttavan luokan nimi
				group.add(item);
				lafMenu.add(item);
			}
			
		}
		return lafMenu;
	}

	
	public JMenuItem getAboutMenuItem() {
		if (aboutMenuItem == null) {
			aboutMenuItem = new JMenuItem("About", KeyEvent.VK_A);
			aboutMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
			
			// Nimetn luokka kuuntelijana
			aboutMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JOptionPane.showMessageDialog(getComponent(), 
							"Swing Component Demo\nCopyright  2006, Ryhm Kalat", 
							"Swing Component Demo", JOptionPane.INFORMATION_MESSAGE);
				}
			});
			
		}
		return aboutMenuItem;
	}

	public JMenuItem getExitMenuItem() {
		if (exitMenuItem == null) {
			// Esimerkki kuinka menuille voidaan asettaa nppimistnavigointi sek ns. pikanppimet
			exitMenuItem = new JMenuItem("Exit", KeyEvent.VK_X);
			exitMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
			
			// nimetn luokka kuuntelijana
			exitMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.exit(0);
				}
			});
		
		}
		
		return exitMenuItem;
	}

	
	
	
	
	public JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			jSplitPane.setTopComponent(getJTabbedPane());
			jSplitPane.setBottomComponent(new JScrollPane(getJTextArea()));
			jSplitPane.setDividerLocation(400);
			jSplitPane.setOneTouchExpandable(true);
		}
		
		return jSplitPane;
	}

	public JTabbedPane getJTabbedPane() {
		if (jTabbedPane == null) {
			jTabbedPane = new JTabbedPane();
			jTabbedPane.addTab("Buttons 'n' stuff", new ButtonsTab(this));
			jTabbedPane.addTab("Text components", new TextComponentTab(this));			
			jTabbedPane.addTab("Tables and Trees", new TableAndTreeTab(this));
			jTabbedPane.addTab("Dialogs", new DialogTab(this));
		}
		
		return jTabbedPane;
	}

	
	
	public JTextArea getJTextArea() {
		if (jTextArea == null) {
			jTextArea = new JTextArea();
			
			// tss listn ponnahdusvalikko, jolla voidaan tyhjen tekstialueen sislt
			jTextArea.setComponentPopupMenu(getPopupMenu());
		}
		
		return jTextArea;
	}

	
	public JPopupMenu getPopupMenu() {
		if (popupMenu == null) {
			popupMenu = new JPopupMenu();
			JMenuItem clear = new JMenuItem("Clear console");

			clear.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					getJTextArea().setText("");
				}
			});
		
			popupMenu.add(clear);
		
		}
		return popupMenu;
	}

	public static void main(String[] args) {
		try {						
			SwingComponents sc = new SwingComponents();
			
		} catch (HeadlessException e) {
			// Tm poikkeus heitetn jos nytt ei ole kytss. Tapahtuu esim. jos etn kynnistetn
			// ohjelma shell konsolista (olettaen ett X ei ole tunneloitu).
			System.out.println("Unable to initialize graphical user interface - no display available");
		}
		
	}

}
