

import java.awt.event.ActionEvent;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.text.MessageFormat;

import javax.swing.AbstractAction;

/**
 * Esimerkki Action-tyyppisest tapahtujakuuntelijasta.
 * Tss hydynnetn Action-tyyppisen kuuntelijan plle- ja poiskytkemist.
 * Eli kun kyttj painaa joko open-nappia taikka enteri tekstikentss, niin
 * tm tapahtuma kytketn pois plt kunnes haluttu sivu on haettu. Tll
 * tavalla saadaan sek tekstikentt, ett nappi kytketty automaattisesti 
 * pois plt.
 * 
 * Koodissa on hydynnetty vapaassa levityksess olevaa SwingWorker luokkaa,
 * joka helpottaa sieohjelmointia Swingin komponenttien kanssa. Swingisshn
 * ptee ns. yhden sikeen snt, joka tarkoittaa sit, ett mitn Swingin
 * komponenttia ei saa pivitt kuin tapahtumasikeest (EventThread) ksin.
 * 
 * SwingWorker luo uuden sikeen, jossa pitkkestoinen tapahtuma ajetaan ja
 * tmn jlkeen kutsuu tapahtumajonon invokeLater -metodia, jossa suoritetaan
 * Swing-komponenttien pivitys. Nin taataan komponenttien virheetn piirto.
 * 
 * SwingWorkerist lytyy juttua esimerkiksi tuolta: 
 * 
 * http://java.sun.com/products/jfc/tsc/articles/threads/threads3.html
 * 
 *
 */
public class OpenAction extends AbstractAction {

	private TinyRSSReader reader;	
	private final MessageFormat errorTemplate;
	
	public OpenAction(TinyRSSReader reader) {
		this.reader = reader;
		// yksinkertainen html template, jota kytetn virheiden tulostamisessa
		this.errorTemplate = new MessageFormat("<html><style>body '{' font-family: arial; margin: 20px;'}'</style><h2>UNABLE TO OPEN RSS DOCUMENT</h2><p>{0}</p>"); 
	}

	public void actionPerformed(ActionEvent e) {
		final String site = reader.getInputPanel().getInput();		
		reader.getStatusPanel().setStatus("Opening "+site, true);
		setEnabled(false);
		
		SwingWorker<String> worker = new SwingWorker<String>() {

			// Tss metodissa haetaan RSS dokumentti ja palautetaan
			// html-muotoinen sislt merkkijonona. Tt metodia
			// suoritetaan omassa sikeess - ei siis tapahtumasikeess.
			@Override
			protected String construct() throws Exception {
				return RSS.getInstance().get(site);
			}	
		
			// Tt metodia kutsutaan kun ylloleva metodi on suoritettu.
			// Tapahtumasie kutsuu tt metodia, joten komponenttien
			// pivittminen tss on turvallista.
			@Override
			protected void finished() {
				
				try {
					reader.getJEditorPane().setText(get());					
					reader.getJEditorPane().setCaretPosition(0);
								
				} catch (Exception e) {
					// tm ei todellakaan ole ntti koodia, mutta muutakaan ei oikein voi, koska
					// get() kapseloi poikkeukset Exception-tyypin sisn ja ne pit kaivaa ulos getCause() metodilla.
					Throwable t = e.getCause();
					
					if (t instanceof IllegalArgumentException || t instanceof MalformedURLException) {
						printError("The URL you provided is invalid.");
					
					} else if (t instanceof UnknownHostException) {
						printError("The given host is unreachable.");
					
					} else {
						printError("Unknown error. It's possible that the given document is not a valid RSS document.");
					}
					
				} finally {
					reader.getStatusPanel().setStatus("Ready", false);
					setEnabled(true);
				}
				
			}
		
			private void printError(String msg) {
				reader.getJEditorPane().setText(errorTemplate.format(new Object[]{msg}));
			}
		};
		
		worker.start();
	
	}

}
