

import java.io.ByteArrayOutputStream;
import java.net.URL;
import java.net.URLConnection;

import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * Yksinkertainen RSS 'parseri', joka kytt XSLT:t muuntamaan
 * haetun RSS dokumentin HTML 3.2:ksi. Luokka on toteutettu kytten
 * Singleton suunnittelumallia.
 * 
 *
 */
public class RSS {

	private final Templates cache;	
	private static RSS rss;
	
	public static synchronized RSS getInstance() throws Exception {
		if (rss == null) {
			rss = new RSS();
		}
	
		return rss;
	}
	
	private RSS() throws Exception {
		// kakutetaan XSLT muunnin, jotta sit ei tarvitse ladata aina uudestaan
		TransformerFactory tfactory = TransformerFactory.newInstance();

		// HUOM: tss kytetn luokkalataajaa lataamaan XSLT kaavain. Tm tarkoittaa sit,
		// ett ohjelma pit paketoida jar:in sisn, jotta tm toimii. Toinen vaihtoehto
		// korvata tm lause esim. nin:
		// cache = tfactory.newTemplates(new StreamSource(new FileInputStream("rss.xsl")));
		// Tietenkin polun pit osoittaa sinne mist rss.xsl lytyy
		cache = tfactory.newTemplates(new StreamSource(getClass().getResourceAsStream("rss.xsl")));
	}
	
	public String get(String site) throws Exception {
		// avataan yhteys haluttuun osoitteeseen
		URL url = new URL(site);
		URLConnection con = url.openConnection();
		con.connect();
		
		// otetaan XSLT muunnin kakusta ja suoritetaan muunnos rss.xsl kaavaimen mukaan
		Transformer transformer = cache.newTransformer();
		ByteArrayOutputStream baos = new ByteArrayOutputStream(8192);
		StreamResult result = new StreamResult(baos);
		transformer.transform(new StreamSource(con.getInputStream()), result); 				
		return baos.toString("UTF-8");	
	}
	
	
}
