

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.border.EmptyBorder;

/**
 * Yksinkertainen RSS lukija, joka lataa RSS dokumentin annetusta
 * osoitteesta ja nytt sen ikkunassa.
 * 
 * 
 *
 */
public class TinyRSSReader extends JFrame {

	private JMenuBar jMenuBar;
	private JMenu fileMenu;
	private JMenuItem aboutMenuItem;
	private JMenuItem exitMenuItem;
	
	private JPanel jContentPane;	
	private InputPanel inputPanel;
	private StatusPanel statusPanel;	
	private JEditorPane jEditorPane;	
	
	private OpenAction openAction;
	
	public TinyRSSReader() {
		super();
		initializeGUI();
	}
	
	private void initializeGUI() {

		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension size = new Dimension(800, 600);		
		this.setLocation( (screen.width - size.width) / 2, (screen.height - size.height) / 2);
		this.setSize(size);
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		this.setTitle("Tiny RSS Reader");
		this.setJMenuBar(getJMenuBar());	
		this.setContentPane(getJContentPane());		
		this.setVisible(true);		
		
	}
	
	public Component getComponent() {
		return this;
	}	
	
	public JMenuBar getJMenuBar() {
		if (jMenuBar == null) {
			jMenuBar = new JMenuBar();
			jMenuBar.add(getFileMenu());
		}
		
		return jMenuBar;
	}	
	
	public JMenu getFileMenu() {
		if (fileMenu == null) {
			fileMenu = new JMenu("File");			
			fileMenu.setMnemonic(KeyEvent.VK_F);
			fileMenu.add(getAboutMenuItem());
			fileMenu.addSeparator();
			fileMenu.add(getExitMenuItem());
		}
		return fileMenu;
	}
	
	public JMenuItem getAboutMenuItem() {
		if (aboutMenuItem == null) {
			aboutMenuItem = new JMenuItem("About", KeyEvent.VK_A);
			aboutMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));		
			aboutMenuItem.addActionListener(new AboutDialogListener());
			
		}
		return aboutMenuItem;
	}

	public JMenuItem getExitMenuItem() {
		if (exitMenuItem == null) {
			exitMenuItem = new JMenuItem("Exit", KeyEvent.VK_X);
			exitMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
			
//			 Esimerkki tapahtumakuuntelijan liittmisest nimettmn sisluokkana
			exitMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.exit(0);
				}
			});
		}	
		return exitMenuItem;
	}
	
	public JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel(new BorderLayout());
			jContentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
			jContentPane.add(getInputPanel(), BorderLayout.NORTH);
			jContentPane.add(new JScrollPane(getJEditorPane()), BorderLayout.CENTER);
			jContentPane.add(getStatusPanel(), BorderLayout.SOUTH);
		
		}
		
		return jContentPane;
	}
	
	public InputPanel getInputPanel() {
		if (inputPanel == null) {
			openAction = new OpenAction(this);
			inputPanel = new InputPanel(openAction);			
		}
		return inputPanel;
	}
	
	public StatusPanel getStatusPanel() {
		if (statusPanel == null) {
			statusPanel = new StatusPanel();
		}
		return statusPanel;
	}

	public JEditorPane getJEditorPane() {
		if (jEditorPane == null) {
			jEditorPane = new JEditorPane("text/html", null);
			jEditorPane.setEditable(false);
		}
		return jEditorPane;
	}

	// Esimerkki tapahtumankuuntelijasta yksityisen sisluokkana
	private class AboutDialogListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JOptionPane.showMessageDialog(getComponent(), 
					"Tiny RSS Reader\nCopyright  2006, Ryhm Kalat", 
					"Tiny RSS Reader", JOptionPane.INFORMATION_MESSAGE);
		}
	}
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			TinyRSSReader reader = new TinyRSSReader();	
		
		} catch (HeadlessException ex) {
			System.out.println("Unable to initialize graphical user interface - no display available");
		}
		
		
	}

}
